@@ -1,7 +1,8 @@
 /*
     device.c -- Interaction BSD tun/tap device
     Copyright (C) 2001-2005 Ivo Timmermans,
-                  2001-2008 Guus Sliepen <guus@tinc-vpn.org>
+                  2001-2009 Guus Sliepen <guus@tinc-vpn.org>
+                  2009      Grzegorz Dymarek <gregd72002@googlemail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,11 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    $Id: device.c 1398 2004-11-01 15:18:53Z guus $
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "system.h"
@@ -27,6 +26,11 @@
 #include "net.h"
 #include "route.h"
 #include "utils.h"
+#include "xalloc.h"
+
+#ifdef HAVE_TUNEMU
+#include "bsd/tunemu.h"
+#endif
 
 #define DEFAULT_DEVICE "/dev/tun0"
 
@@ -34,15 +38,20 @@ typedef enum device_type {
 	DEVICE_TYPE_TUN,
 	DEVICE_TYPE_TUNIFHEAD,
 	DEVICE_TYPE_TAP,
+#ifdef HAVE_TUNEMU
+	DEVICE_TYPE_TUNEMU,
+#endif
 } device_type_t;
 
 int device_fd = -1;
-char *device;
-char *iface;
-char *device_info;
+char *device = NULL;
+char *iface = NULL;
+static char *device_info = NULL;
 static int device_total_in = 0;
 static int device_total_out = 0;
-#if defined(HAVE_OPENBSD) || defined(HAVE_FREEBSD)
+#if defined(TUNEMU)
+static device_type_t device_type = DEVICE_TYPE_TUNEMU;
+#elif defined(HAVE_OPENBSD) || defined(HAVE_FREEBSD)
 static device_type_t device_type = DEVICE_TYPE_TUNIFHEAD;
 #else
 static device_type_t device_type = DEVICE_TYPE_TUN;
@@ -51,22 +60,19 @@ static device_type_t device_type = DEVICE_TYPE_TUN;
 bool setup_device(void) {
 	char *type;
 
-	cp();
-
 	if(!get_config_string(lookup_config(config_tree, "Device"), &device))
-		device = DEFAULT_DEVICE;
+		device = xstrdup(DEFAULT_DEVICE);
 
 	if(!get_config_string(lookup_config(config_tree, "Interface"), &iface))
-		iface = rindex(device, '/') ? rindex(device, '/') + 1 : device;
-
-	if((device_fd = open(device, O_RDWR | O_NONBLOCK)) < 0) {
-		logger(LOG_ERR, _("Could not open %s: %s"), device, strerror(errno));
-		return false;
-	}
+		iface = xstrdup(rindex(device, '/') ? rindex(device, '/') + 1 : device);
 
 	if(get_config_string(lookup_config(config_tree, "DeviceType"), &type)) {
 		if(!strcasecmp(type, "tun"))
 			/* use default */;	
+#ifdef HAVE_TUNEMU
+		else if(!strcasecmp(type, "tunemu"))
+			device_type = DEVICE_TYPE_TUNEMU;
+#endif
 		else if(!strcasecmp(type, "tunnohead"))
 			device_type = DEVICE_TYPE_TUN;
 		else if(!strcasecmp(type, "tunifhead"))
@@ -74,7 +80,7 @@ bool setup_device(void) {
 		else if(!strcasecmp(type, "tap"))
 			device_type = DEVICE_TYPE_TAP;
 		else {
-			logger(LOG_ERR, _("Unknown device type %s!"), type);
+			logger(LOG_ERR, "Unknown device type %s!", type);
 			return false;
 		}
 	} else {
@@ -82,6 +88,23 @@ bool setup_device(void) {
 			device_type = DEVICE_TYPE_TAP;
 	}
 
+	switch(device_type) {
+#ifdef HAVE_TUNEMU
+		case DEVICE_TYPE_TUNEMU: {
+			char dynamic_name[256] = "";
+        		device_fd = tunemu_open(dynamic_name);
+		}
+			break;
+#endif
+		default:
+			device_fd = open(device, O_RDWR | O_NONBLOCK);
+	}
+
+	if(device_fd < 0) {
+		logger(LOG_ERR, "Could not open %s: %s", device, strerror(errno));
+		return false;
+	}
+
 	switch(device_type) {
 		default:
 			device_type = DEVICE_TYPE_TUN;
@@ -90,7 +113,7 @@ bool setup_device(void) {
 		{	
 			const int zero = 0;
 			if(ioctl(device_fd, TUNSIFHEAD, &zero, sizeof zero) == -1) {
-				logger(LOG_ERR, _("System call `%s' failed: %s"), "ioctl", strerror(errno));
+				logger(LOG_ERR, "System call `%s' failed: %s", "ioctl", strerror(errno));
 				return false;
 			}
 		}
@@ -102,14 +125,14 @@ bool setup_device(void) {
 		}
 #endif
 
-			device_info = _("Generic BSD tun device");
+			device_info = "Generic BSD tun device";
 			break;
 		case DEVICE_TYPE_TUNIFHEAD:
 #ifdef TUNSIFHEAD
 		{
 			const int one = 1;
 			if(ioctl(device_fd, TUNSIFHEAD, &one, sizeof one) == -1) {
-				logger(LOG_ERR, _("System call `%s' failed: %s"), "ioctl", strerror(errno));
+				logger(LOG_ERR, "System call `%s' failed: %s", "ioctl", strerror(errno));
 				return false;
 			}
 		}
@@ -121,35 +144,67 @@ bool setup_device(void) {
 		}
 #endif
 
-			device_info = _("Generic BSD tun device");
+			device_info = "Generic BSD tun device";
 			break;
 		case DEVICE_TYPE_TAP:
 			if(routing_mode == RMODE_ROUTER)
 				overwrite_mac = true;
-			device_info = _("Generic BSD tap device");
+			device_info = "Generic BSD tap device";
+#ifdef TAPGIFNAME
+			{
+				struct ifreq ifr;
+				if(ioctl(device_fd, TAPGIFNAME, (void*)&ifr) == 0) {
+					if(iface)
+						free(iface);
+					iface = xstrdup(ifr.ifr_name);
+				}
+			}
+			
+#endif
 			break;
+#ifdef HAVE_TUNEMU
+		case DEVICE_TYPE_TUNEMU:
+        		device_info = "BSD tunemu device";
+			break;
+#endif
 	}
 
-	logger(LOG_INFO, _("%s is a %s"), device, device_info);
+	logger(LOG_INFO, "%s is a %s", device, device_info);
 
 	return true;
 }
 
 void close_device(void) {
-	cp();
+	switch(device_type) {
+#ifdef HAVE_TUNEMU
+		case DEVICE_TYPE_TUNEMU:
+			tunemu_close(device_fd);
+			break;
+#endif
+		default:
+			close(device_fd);
+	}
 
-	close(device_fd);
+	free(device);
+	free(iface);
 }
 
 bool read_packet(vpn_packet_t *packet) {
 	int lenin;
 
-	cp();
-
 	switch(device_type) {
 		case DEVICE_TYPE_TUN:
-			if((lenin = read(device_fd, packet->data + 14, MTU - 14)) <= 0) {
-				logger(LOG_ERR, _("Error while reading from %s %s: %s"), device_info,
+#ifdef HAVE_TUNEMU
+		case DEVICE_TYPE_TUNEMU:
+			if(device_type == DEVICE_TYPE_TUNEMU)
+				lenin = tunemu_read(device_fd, packet->data + 14, MTU - 14);
+			else
+#else
+				lenin = read(device_fd, packet->data + 14, MTU - 14);
+#endif
+
+			if(lenin <= 0) {
+				logger(LOG_ERR, "Error while reading from %s %s: %s", device_info,
 					   device, strerror(errno));
 				return false;
 			}
@@ -165,7 +220,7 @@ bool read_packet(vpn_packet_t *packet) {
 					break;
 				default:
 					ifdebug(TRAFFIC) logger(LOG_ERR,
-							   _ ("Unknown IP version %d while reading packet from %s %s"),
+							   "Unknown IP version %d while reading packet from %s %s",
 							   packet->data[14] >> 4, device_info, device);
 					return false;
 			}
@@ -178,7 +233,7 @@ bool read_packet(vpn_packet_t *packet) {
 			struct iovec vector[2] = {{&type, sizeof(type)}, {packet->data + 14, MTU - 14}};
 
 			if((lenin = readv(device_fd, vector, 2)) <= 0) {
-				logger(LOG_ERR, _("Error while reading from %s %s: %s"), device_info,
+				logger(LOG_ERR, "Error while reading from %s %s: %s", device_info,
 					   device, strerror(errno));
 				return false;
 			}
@@ -196,7 +251,7 @@ bool read_packet(vpn_packet_t *packet) {
 
 				default:
 					ifdebug(TRAFFIC) logger(LOG_ERR,
-							   _ ("Unknown address family %x while reading packet from %s %s"),
+							   "Unknown address family %x while reading packet from %s %s",
 							   ntohl(type), device_info, device);
 					return false;
 			}
@@ -207,7 +262,7 @@ bool read_packet(vpn_packet_t *packet) {
 
 		case DEVICE_TYPE_TAP:
 			if((lenin = read(device_fd, packet->data, MTU)) <= 0) {
-				logger(LOG_ERR, _("Error while reading from %s %s: %s"), device_info,
+				logger(LOG_ERR, "Error while reading from %s %s: %s", device_info,
 					   device, strerror(errno));
 				return false;
 			}
@@ -221,23 +276,20 @@ bool read_packet(vpn_packet_t *packet) {
 		
 	device_total_in += packet->len;
 
-	ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Read packet of %d bytes from %s"),
+	ifdebug(TRAFFIC) logger(LOG_DEBUG, "Read packet of %d bytes from %s",
 			   packet->len, device_info);
 
 	return true;
 }
 
-bool write_packet(vpn_packet_t *packet)
-{
-	cp();
-
-	ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Writing packet of %d bytes to %s"),
+bool write_packet(vpn_packet_t *packet) {
+	ifdebug(TRAFFIC) logger(LOG_DEBUG, "Writing packet of %d bytes to %s",
 			   packet->len, device_info);
 
 	switch(device_type) {
 		case DEVICE_TYPE_TUN:
 			if(write(device_fd, packet->data + 14, packet->len - 14) < 0) {
-				logger(LOG_ERR, _("Error while writing to %s %s: %s"), device_info,
+				logger(LOG_ERR, "Error while writing to %s %s: %s", device_info,
 					   device, strerror(errno));
 				return false;
 			}
@@ -259,13 +311,13 @@ bool write_packet(vpn_packet_t *packet)
 					break;
 				default:
 					ifdebug(TRAFFIC) logger(LOG_ERR,
-							   _("Unknown address family %x while writing packet to %s %s"),
+							   "Unknown address family %x while writing packet to %s %s",
 							   af, device_info, device);
 					return false;
 			}
 
 			if(writev(device_fd, vector, 2) < 0) {
-				logger(LOG_ERR, _("Can't write to %s %s: %s"), device_info, device,
+				logger(LOG_ERR, "Can't write to %s %s: %s", device_info, device,
 					   strerror(errno));
 				return false;
 			}
@@ -274,12 +326,22 @@ bool write_packet(vpn_packet_t *packet)
 			
 		case DEVICE_TYPE_TAP:
 			if(write(device_fd, packet->data, packet->len) < 0) {
-				logger(LOG_ERR, _("Error while writing to %s %s: %s"), device_info,
+				logger(LOG_ERR, "Error while writing to %s %s: %s", device_info,
 					   device, strerror(errno));
 				return false;
 			}
 			break;
 
+#ifdef HAVE_TUNEMU
+		case DEVICE_TYPE_TUNEMU:
+			if(tunemu_write(device_fd, packet->data + 14, packet->len - 14) < 0) {
+				logger(LOG_ERR, "Error while writing to %s %s: %s", device_info,
+					   device, strerror(errno));
+				return false;
+			}
+			break;
+#endif
+
 		default:
 			return false;
 	}
@@ -289,11 +351,8 @@ bool write_packet(vpn_packet_t *packet)
 	return true;
 }
 
-void dump_device_stats(void)
-{
-	cp();
-
-	logger(LOG_DEBUG, _("Statistics for %s %s:"), device_info, device);
-	logger(LOG_DEBUG, _(" total bytes in:  %10d"), device_total_in);
-	logger(LOG_DEBUG, _(" total bytes out: %10d"), device_total_out);
+void dump_device_stats(void) {
+	logger(LOG_DEBUG, "Statistics for %s %s:", device_info, device);
+	logger(LOG_DEBUG, " total bytes in:  %10d", device_total_in);
+	logger(LOG_DEBUG, " total bytes out: %10d", device_total_out);
 }
