@@ -130,11 +130,11 @@ void dump_connections(void) {
 
 bool read_connection_config(connection_t *c) {
 	char *fname;
-	int x;
+	bool x;
 
 	xasprintf(&fname, "%s/hosts/%s", confbase, c->name);
 	x = read_config_file(c->config_tree, fname);
 	free(fname);
 
-	return x == 0;
+	return x;
 }
