@@ -1,7 +1,7 @@
 /*
     device.c -- Interaction with Windows tap driver in a MinGW environment
     Copyright (C) 2002-2005 Ivo Timmermans,
-                  2002-2007 Guus Sliepen <guus@tinc-vpn.org>
+                  2002-2009 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,11 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    $Id: device.c 1510 2007-05-16 14:46:25Z guus $
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "system.h"
@@ -34,74 +32,35 @@
 
 #include "mingw/common.h"
 
-int device_fd = 0;
+int device_fd = -1;
 static HANDLE device_handle = INVALID_HANDLE_VALUE;
 char *device = NULL;
 char *iface = NULL;
-char *device_info = NULL;
+static char *device_info = NULL;
 
 static int device_total_in = 0;
 static int device_total_out = 0;
 
 extern char *myport;
 
-static struct packetbuf {
-	uint8_t data[MTU];
-	length_t len;
-} *bufs;
-
-static int nbufs = 64;
-
-DWORD WINAPI tapreader(void *bla) {
-	int sock, err, status;
-	struct addrinfo *ai;
-	struct addrinfo hint = {
-		.ai_family = AF_UNSPEC,
-		.ai_socktype = SOCK_STREAM,
-		.ai_protocol = IPPROTO_TCP,
-		.ai_flags = 0,
-	};
-	unsigned char bufno = 0;
+static DWORD WINAPI tapreader(void *bla) {
+	int status;
 	long len;
 	OVERLAPPED overlapped;
+	vpn_packet_t packet;
 
-	/* Open a socket to the parent process */
-
-	err = getaddrinfo(NULL, myport, &hint, &ai);
-
-	if(err || !ai) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "getaddrinfo", gai_strerror(errno));
-		return -1;
-	}
-
-	sock = socket(ai->ai_family, SOCK_STREAM, IPPROTO_TCP);
-
-	if(sock < 0) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "socket", strerror(errno));
-		freeaddrinfo(ai);
-		return -1;
-	}
-
-	if(connect(sock, ai->ai_addr, ai->ai_addrlen)) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "connect", strerror(errno));
-		freeaddrinfo(ai);
-		return -1;
-	}
-
-	freeaddrinfo(ai);
-
-	logger(LOG_DEBUG, _("Tap reader running"));
+	logger(LOG_DEBUG, "Tap reader running");
 
 	/* Read from tap device and send to parent */
 
 	overlapped.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
-	
+
 	for(;;) {
 		overlapped.Offset = 0;
 		overlapped.OffsetHigh = 0;
 		ResetEvent(overlapped.hEvent);
 
-		status = ReadFile(device_handle, bufs[bufno].data, MTU, &len, &overlapped);
+		status = ReadFile(device_handle, packet.data, MTU, &len, &overlapped);
 
 		if(!status) {
 			if(GetLastError() == ERROR_IO_PENDING) {
@@ -109,22 +68,21 @@ DWORD WINAPI tapreader(void *bla) {
 				if(!GetOverlappedResult(device_handle, &overlapped, &len, FALSE))
 					continue;
 			} else {
-				logger(LOG_ERR, _("Error while reading from %s %s: %s"), device_info,
+				logger(LOG_ERR, "Error while reading from %s %s: %s", device_info,
 					   device, strerror(errno));
 				return -1;
 			}
 		}
 
-		bufs[bufno].len = len;
-		if(send(sock, &bufno, 1, 0) <= 0)
-			return -1;
-		if(++bufno >= nbufs)
-			bufno = 0;
+		EnterCriticalSection(&mutex);
+		packet.len = len;
+		packet.priority = 0;
+		route(myself, &packet);
+		LeaveCriticalSection(&mutex);
 	}
 }
 
-bool setup_device(void)
-{
+bool setup_device(void) {
 	HKEY key, key2;
 	int i;
 
@@ -148,15 +106,13 @@ bool setup_device(void)
 		.ai_flags = 0,
 	};
 
-	cp();
-
 	get_config_string(lookup_config(config_tree, "Device"), &device);
 	get_config_string(lookup_config(config_tree, "Interface"), &iface);
 
 	/* Open registry and look for network adapters */
 
 	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, NETWORK_CONNECTIONS_KEY, 0, KEY_READ, &key)) {
-		logger(LOG_ERR, _("Unable to read registry: %s"), winerror(GetLastError()));
+		logger(LOG_ERR, "Unable to read registry: %s", winerror(GetLastError()));
 		return false;
 	}
 
@@ -207,7 +163,7 @@ bool setup_device(void)
 	RegCloseKey(key);
 
 	if(!found) {
-		logger(LOG_ERR, _("No Windows tap device found!"));
+		logger(LOG_ERR, "No Windows tap device found!");
 		return false;
 	}
 
@@ -225,14 +181,14 @@ bool setup_device(void)
 	}
 	
 	if(device_handle == INVALID_HANDLE_VALUE) {
-		logger(LOG_ERR, _("%s (%s) is not a usable Windows tap device: %s"), device, iface, winerror(GetLastError()));
+		logger(LOG_ERR, "%s (%s) is not a usable Windows tap device: %s", device, iface, winerror(GetLastError()));
 		return false;
 	}
 
 	/* Get MAC address from tap device */
 
 	if(!DeviceIoControl(device_handle, TAP_IOCTL_GET_MAC, mymac.x, sizeof(mymac.x), mymac.x, sizeof(mymac.x), &len, 0)) {
-		logger(LOG_ERR, _("Could not get MAC address from Windows tap device %s (%s): %s"), device, iface, winerror(GetLastError()));
+		logger(LOG_ERR, "Could not get MAC address from Windows tap device %s (%s): %s", device, iface, winerror(GetLastError()));
 		return false;
 	}
 
@@ -240,116 +196,47 @@ bool setup_device(void)
 		overwrite_mac = 1;
 	}
 
-	/* Set up ringbuffer */
-
-	get_config_int(lookup_config(config_tree, "RingBufferSize"), &nbufs);
-	if(nbufs <= 1)
-		nbufs = 1;
-	else if(nbufs > 256)
-		nbufs = 256;
-	
-	bufs = xmalloc_and_zero(nbufs * sizeof *bufs);
-
-	/* Create a listening socket */
-
-	err = getaddrinfo(NULL, myport, &hint, &ai);
-
-	if(err || !ai) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "getaddrinfo", gai_strerror(errno));
-		return false;
-	}
-
-	sock = socket(ai->ai_family, SOCK_STREAM, IPPROTO_TCP);
-
-	if(sock < 0) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "socket", strerror(errno));
-		return false;
-	}
-
-	if(bind(sock, ai->ai_addr, ai->ai_addrlen)) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "bind", strerror(errno));
-		return false;
-	}
-
-	freeaddrinfo(ai);
-
-	if(listen(sock, 1)) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "listen", strerror(errno));
-		return false;
-	}
-
 	/* Start the tap reader */
 
 	thread = CreateThread(NULL, 0, tapreader, NULL, 0, NULL);
 
 	if(!thread) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "CreateThread", winerror(GetLastError()));
+		logger(LOG_ERR, "System call `%s' failed: %s", "CreateThread", winerror(GetLastError()));
 		return false;
 	}
 
-	/* Wait for the tap reader to connect back to us */
-
-	if((device_fd = accept(sock, NULL, 0)) == -1) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "accept", strerror(errno));
-		return false;
-	}
-
-	closesocket(sock);
-
 	/* Set media status for newer TAP-Win32 devices */
 
 	status = true;
 	DeviceIoControl(device_handle, TAP_IOCTL_SET_MEDIA_STATUS, &status, sizeof(status), &status, sizeof(status), &len, NULL);
 
-	device_info = _("Windows tap device");
+	device_info = "Windows tap device";
 
-	logger(LOG_INFO, _("%s (%s) is a %s"), device, iface, device_info);
+	logger(LOG_INFO, "%s (%s) is a %s", device, iface, device_info);
 
 	return true;
 }
 
-void close_device(void)
-{
-	cp();
-
+void close_device(void) {
 	CloseHandle(device_handle);
+
+	free(device);
+	free(iface);
 }
 
-bool read_packet(vpn_packet_t *packet)
-{
-	unsigned char bufno;
-
-	cp();
-
-	if((recv(device_fd, &bufno, 1, 0)) <= 0) {
-		logger(LOG_ERR, _("Error while reading from %s %s: %s"), device_info,
-			   device, strerror(errno));
-		return false;
-	}
-	
-	packet->len = bufs[bufno].len;
-	memcpy(packet->data, bufs[bufno].data, bufs[bufno].len);
-
-	device_total_in += packet->len;
-
-	ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Read packet of %d bytes from %s"), packet->len,
-			   device_info);
-
-	return true;
+bool read_packet(vpn_packet_t *packet) {
+	return false;
 }
 
-bool write_packet(vpn_packet_t *packet)
-{
+bool write_packet(vpn_packet_t *packet) {
 	long lenout;
 	OVERLAPPED overlapped = {0};
 
-	cp();
-
-	ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Writing packet of %d bytes to %s"),
+	ifdebug(TRAFFIC) logger(LOG_DEBUG, "Writing packet of %d bytes to %s",
 			   packet->len, device_info);
 
 	if(!WriteFile(device_handle, packet->data, packet->len, &lenout, &overlapped)) {
-		logger(LOG_ERR, _("Error while writing to %s %s: %s"), device_info, device, winerror(GetLastError()));
+		logger(LOG_ERR, "Error while writing to %s %s: %s", device_info, device, winerror(GetLastError()));
 		return false;
 	}
 
@@ -358,11 +245,8 @@ bool write_packet(vpn_packet_t *packet)
 	return true;
 }
 
-void dump_device_stats(void)
-{
-	cp();
-
-	logger(LOG_DEBUG, _("Statistics for %s %s:"), device_info, device);
-	logger(LOG_DEBUG, _(" total bytes in:  %10d"), device_total_in);
-	logger(LOG_DEBUG, _(" total bytes out: %10d"), device_total_out);
+void dump_device_stats(void) {
+	logger(LOG_DEBUG, "Statistics for %s %s:", device_info, device);
+	logger(LOG_DEBUG, " total bytes in:  %10d", device_total_in);
+	logger(LOG_DEBUG, " total bytes out: %10d", device_total_out);
 }
