@@ -1,7 +1,7 @@
 /*
     process.c -- process management functions
     Copyright (C) 1999-2005 Ivo Timmermans,
-                  2000-2007 Guus Sliepen <guus@tinc-vpn.org>
+                  2000-2009 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,11 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    $Id: process.c 1595 2008-12-22 20:27:52Z guus $
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "system.h"
@@ -49,10 +47,8 @@ sigset_t emptysigset;
 
 static int saved_debug_level = -1;
 
-static void memory_full(int size)
-{
-	logger(LOG_ERR, _("Memory exhausted (couldn't allocate %d bytes), exitting."), size);
-	cp_trace();
+static void memory_full(int size) {
+	logger(LOG_ERR, "Memory exhausted (couldn't allocate %d bytes), exitting.", size);
 	exit(1);
 }
 
@@ -76,7 +72,7 @@ bool install_service(void) {
 
 	manager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
 	if(!manager) {
-		logger(LOG_ERR, _("Could not open service manager: %s"), winerror(GetLastError()));
+		logger(LOG_ERR, "Could not open service manager: %s", winerror(GetLastError()));
 		return false;
 	}
 
@@ -107,18 +103,23 @@ bool install_service(void) {
 			command, NULL, NULL, NULL, NULL, NULL);
 	
 	if(!service) {
-		logger(LOG_ERR, _("Could not create %s service: %s"), identname, winerror(GetLastError()));
-		return false;
+		DWORD lasterror = GetLastError();
+		logger(LOG_ERR, "Could not create %s service: %s", identname, winerror(lasterror));
+		if(lasterror != ERROR_SERVICE_EXISTS)
+			return false;
 	}
 
-	ChangeServiceConfig2(service, SERVICE_CONFIG_DESCRIPTION, &description);
-
-	logger(LOG_INFO, _("%s service installed"), identname);
+	if(service) {
+		ChangeServiceConfig2(service, SERVICE_CONFIG_DESCRIPTION, &description);
+		logger(LOG_INFO, "%s service installed", identname);
+	} else {
+		service = OpenService(manager, identname, SERVICE_ALL_ACCESS);
+	}
 
 	if(!StartService(service, 0, NULL))
-		logger(LOG_WARNING, _("Could not start %s service: %s"), identname, winerror(GetLastError()));
+		logger(LOG_WARNING, "Could not start %s service: %s", identname, winerror(GetLastError()));
 	else
-		logger(LOG_INFO, _("%s service started"), identname);
+		logger(LOG_INFO, "%s service started", identname);
 
 	return true;
 }
@@ -126,28 +127,28 @@ bool install_service(void) {
 bool remove_service(void) {
 	manager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
 	if(!manager) {
-		logger(LOG_ERR, _("Could not open service manager: %s"), winerror(GetLastError()));
+		logger(LOG_ERR, "Could not open service manager: %s", winerror(GetLastError()));
 		return false;
 	}
 
 	service = OpenService(manager, identname, SERVICE_ALL_ACCESS);
 
 	if(!service) {
-		logger(LOG_ERR, _("Could not open %s service: %s"), identname, winerror(GetLastError()));
+		logger(LOG_ERR, "Could not open %s service: %s", identname, winerror(GetLastError()));
 		return false;
 	}
 
 	if(!ControlService(service, SERVICE_CONTROL_STOP, &status))
-		logger(LOG_ERR, _("Could not stop %s service: %s"), identname, winerror(GetLastError()));
+		logger(LOG_ERR, "Could not stop %s service: %s", identname, winerror(GetLastError()));
 	else
-		logger(LOG_INFO, _("%s service stopped"), identname);
+		logger(LOG_INFO, "%s service stopped", identname);
 
 	if(!DeleteService(service)) {
-		logger(LOG_ERR, _("Could not remove %s service: %s"), identname, winerror(GetLastError()));
+		logger(LOG_ERR, "Could not remove %s service: %s", identname, winerror(GetLastError()));
 		return false;
 	}
 
-	logger(LOG_INFO, _("%s service removed"), identname);
+	logger(LOG_INFO, "%s service removed", identname);
 
 	return true;
 }
@@ -158,13 +159,13 @@ DWORD WINAPI controlhandler(DWORD request, DWORD type, LPVOID boe, LPVOID bah) {
 			SetServiceStatus(statushandle, &status);
 			return NO_ERROR;
 		case SERVICE_CONTROL_STOP:
-			logger(LOG_NOTICE, _("Got %s request"), "SERVICE_CONTROL_STOP");
+			logger(LOG_NOTICE, "Got %s request", "SERVICE_CONTROL_STOP");
 			break;
 		case SERVICE_CONTROL_SHUTDOWN:
-			logger(LOG_NOTICE, _("Got %s request"), "SERVICE_CONTROL_SHUTDOWN");
+			logger(LOG_NOTICE, "Got %s request", "SERVICE_CONTROL_SHUTDOWN");
 			break;
 		default:
-			logger(LOG_WARNING, _("Got unexpected request %d"), request);
+			logger(LOG_WARNING, "Got unexpected request %d", request);
 			return ERROR_CALL_NOT_IMPLEMENTED;
 	}
 
@@ -183,8 +184,7 @@ DWORD WINAPI controlhandler(DWORD request, DWORD type, LPVOID boe, LPVOID bah) {
 
 }
 
-VOID WINAPI run_service(DWORD argc, LPTSTR* argv)
-{
+VOID WINAPI run_service(DWORD argc, LPTSTR* argv) {
 	int err = 1;
 	extern int main2(int argc, char **argv);
 
@@ -198,7 +198,7 @@ VOID WINAPI run_service(DWORD argc, LPTSTR* argv)
 	statushandle = RegisterServiceCtrlHandlerEx(identname, controlhandler, NULL); 
 
 	if (!statushandle) {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "RegisterServiceCtrlHandlerEx", winerror(GetLastError()));
+		logger(LOG_ERR, "System call `%s' failed: %s", "RegisterServiceCtrlHandlerEx", winerror(GetLastError()));
 		err = 1;
 	} else {
 		status.dwWaitHint = 30000; 
@@ -231,7 +231,7 @@ bool init_service(void) {
 			return false;
 		}
 		else
-			logger(LOG_ERR, _("System call `%s' failed: %s"), "StartServiceCtrlDispatcher", winerror(GetLastError()));
+			logger(LOG_ERR, "System call `%s' failed: %s", "StartServiceCtrlDispatcher", winerror(GetLastError()));
 	}
 
 	return true;
@@ -242,26 +242,23 @@ bool init_service(void) {
 /*
   check for an existing tinc for this net, and write pid to pidfile
 */
-static bool write_pidfile(void)
-{
+static bool write_pidfile(void) {
 	pid_t pid;
 
-	cp();
-
 	pid = check_pid(pidfilename);
 
 	if(pid) {
 		if(netname)
-			fprintf(stderr, _("A tincd is already running for net `%s' with pid %ld.\n"),
+			fprintf(stderr, "A tincd is already running for net `%s' with pid %ld.\n",
 					netname, (long)pid);
 		else
-			fprintf(stderr, _("A tincd is already running with pid %ld.\n"), (long)pid);
+			fprintf(stderr, "A tincd is already running with pid %ld.\n", (long)pid);
 		return false;
 	}
 
 	/* if it's locked, write-protected, or whatever */
 	if(!write_pid(pidfilename)) {
-		fprintf(stderr, _("Could write pid file %s: %s\n"), pidfilename, strerror(errno));
+		fprintf(stderr, "Could write pid file %s: %s\n", pidfilename, strerror(errno));
 		return false;
 	}
 
@@ -272,21 +269,18 @@ static bool write_pidfile(void)
 /*
   kill older tincd for this net
 */
-bool kill_other(int signal)
-{
+bool kill_other(int signal) {
 #ifndef HAVE_MINGW
 	pid_t pid;
 
-	cp();
-
 	pid = read_pid(pidfilename);
 
 	if(!pid) {
 		if(netname)
-			fprintf(stderr, _("No other tincd is running for net `%s'.\n"),
+			fprintf(stderr, "No other tincd is running for net `%s'.\n",
 					netname);
 		else
-			fprintf(stderr, _("No other tincd is running.\n"));
+			fprintf(stderr, "No other tincd is running.\n");
 		return false;
 	}
 
@@ -295,12 +289,12 @@ bool kill_other(int signal)
 	/* ESRCH is returned when no process with that pid is found */
 	if(kill(pid, signal) && errno == ESRCH) {
 		if(netname)
-			fprintf(stderr, _("The tincd for net `%s' is no longer running. "),
+			fprintf(stderr, "The tincd for net `%s' is no longer running. ",
 					netname);
 		else
-			fprintf(stderr, _("The tincd is no longer running. "));
+			fprintf(stderr, "The tincd is no longer running. ");
 
-		fprintf(stderr, _("Removing stale lock file.\n"));
+		fprintf(stderr, "Removing stale lock file.\n");
 		remove_pid(pidfilename);
 	}
 
@@ -313,10 +307,7 @@ bool kill_other(int signal)
 /*
   Detach from current terminal, write pidfile, kill parent
 */
-bool detach(void)
-{
-	cp();
-
+bool detach(void) {
 	setup_signals();
 
 	/* First check if we can open a fresh new pidfile */
@@ -333,7 +324,7 @@ bool detach(void)
 	if(do_detach) {
 #ifndef HAVE_MINGW
 		if(daemon(0, 0)) {
-			fprintf(stderr, _("Couldn't detach from terminal: %s"),
+			fprintf(stderr, "Couldn't detach from terminal: %s",
 					strerror(errno));
 			return false;
 		}
@@ -341,7 +332,7 @@ bool detach(void)
 		/* Now UPDATE the pid in the pidfile, because we changed it... */
 
 		if(!write_pid(pidfilename)) {
-			fprintf(stderr, _("Could not write pid file %s: %s\n"), pidfilename, strerror(errno));
+			fprintf(stderr, "Could not write pid file %s: %s\n", pidfilename, strerror(errno));
 			return false;
 		}
 #else
@@ -352,7 +343,7 @@ bool detach(void)
 
 	openlogger(identname, use_logfile?LOGMODE_FILE:(do_detach?LOGMODE_SYSLOG:LOGMODE_STDERR));
 
-	logger(LOG_NOTICE, _("tincd %s (%s %s) starting, debug level %d"),
+	logger(LOG_NOTICE, "tincd %s (%s %s) starting, debug level %d",
 			   VERSION, __DATE__, __TIME__, debug_level);
 
 	xalloc_fail_func = memory_full;
@@ -360,34 +351,32 @@ bool detach(void)
 	return true;
 }
 
-bool execute_script(const char *name, char **envp)
-{
+bool execute_script(const char *name, char **envp) {
 #ifdef HAVE_SYSTEM
 	int status, len;
-	struct stat s;
 	char *scriptname, *p;
 	int i;
 
-	cp();
-
 #ifndef HAVE_MINGW
-	len = asprintf(&scriptname, "\"%s/%s\"", confbase, name);
+	len = xasprintf(&scriptname, "\"%s/%s\"", confbase, name);
 #else
-	len = asprintf(&scriptname, "\"%s/%s.bat\"", confbase, name);
+	len = xasprintf(&scriptname, "\"%s/%s.bat\"", confbase, name);
 #endif
 	if(len < 0)
 		return false;
 
 	scriptname[len - 1] = '\0';
 
+#ifndef HAVE_TUNEMU
 	/* First check if there is a script */
 
-	if(stat(scriptname + 1, &s)) {
+	if(access(scriptname + 1, F_OK)) {
 		free(scriptname);
 		return true;
 	}
+#endif
 
-	ifdebug(STATUS) logger(LOG_INFO, _("Executing script %s"), name);
+	ifdebug(STATUS) logger(LOG_INFO, "Executing script %s", name);
 
 #ifdef HAVE_PUTENV
 	/* Set environment */
@@ -417,20 +406,20 @@ bool execute_script(const char *name, char **envp)
 	if(status != -1) {
 		if(WIFEXITED(status)) {	/* Child exited by itself */
 			if(WEXITSTATUS(status)) {
-				logger(LOG_ERR, _("Script %s exited with non-zero status %d"),
+				logger(LOG_ERR, "Script %s exited with non-zero status %d",
 					   name, WEXITSTATUS(status));
 				return false;
 			}
 		} else if(WIFSIGNALED(status)) {	/* Child was killed by a signal */
-			logger(LOG_ERR, _("Script %s was killed by signal %d (%s)"),
+			logger(LOG_ERR, "Script %s was killed by signal %d (%s)",
 				   name, WTERMSIG(status), strsignal(WTERMSIG(status)));
 			return false;
 		} else {			/* Something strange happened */
-			logger(LOG_ERR, _("Script %s terminated abnormally"), name);
+			logger(LOG_ERR, "Script %s terminated abnormally", name);
 			return false;
 		}
 	} else {
-		logger(LOG_ERR, _("System call `%s' failed: %s"), "system", strerror(errno));
+		logger(LOG_ERR, "System call `%s' failed: %s", "system", strerror(errno));
 		return false;
 	}
 #endif
@@ -444,40 +433,34 @@ bool execute_script(const char *name, char **envp)
 */
 
 #ifndef HAVE_MINGW
-static RETSIGTYPE sigterm_handler(int a)
-{
-	logger(LOG_NOTICE, _("Got %s signal"), "TERM");
+static RETSIGTYPE sigterm_handler(int a) {
+	logger(LOG_NOTICE, "Got %s signal", "TERM");
 	if(running)
 		running = false;
 	else
 		exit(1);
 }
 
-static RETSIGTYPE sigquit_handler(int a)
-{
-	logger(LOG_NOTICE, _("Got %s signal"), "QUIT");
+static RETSIGTYPE sigquit_handler(int a) {
+	logger(LOG_NOTICE, "Got %s signal", "QUIT");
 	if(running)
 		running = false;
 	else
 		exit(1);
 }
 
-static RETSIGTYPE fatal_signal_square(int a)
-{
-	logger(LOG_ERR, _("Got another fatal signal %d (%s): not restarting."), a,
+static RETSIGTYPE fatal_signal_square(int a) {
+	logger(LOG_ERR, "Got another fatal signal %d (%s): not restarting.", a,
 		   strsignal(a));
-	cp_trace();
 	exit(1);
 }
 
-static RETSIGTYPE fatal_signal_handler(int a)
-{
+static RETSIGTYPE fatal_signal_handler(int a) {
 	struct sigaction act;
-	logger(LOG_ERR, _("Got fatal signal %d (%s)"), a, strsignal(a));
-	cp_trace();
+	logger(LOG_ERR, "Got fatal signal %d (%s)", a, strsignal(a));
 
 	if(do_detach) {
-		logger(LOG_NOTICE, _("Trying to re-execute in 5 seconds..."));
+		logger(LOG_NOTICE, "Trying to re-execute in 5 seconds...");
 
 		act.sa_handler = fatal_signal_square;
 		act.sa_mask = emptysigset;
@@ -489,68 +472,59 @@ static RETSIGTYPE fatal_signal_handler(int a)
 		remove_pid(pidfilename);
 		execvp(g_argv[0], g_argv);
 	} else {
-		logger(LOG_NOTICE, _("Not restarting."));
+		logger(LOG_NOTICE, "Not restarting.");
 		exit(1);
 	}
 }
 
-static RETSIGTYPE sighup_handler(int a)
-{
-	logger(LOG_NOTICE, _("Got %s signal"), "HUP");
+static RETSIGTYPE sighup_handler(int a) {
+	logger(LOG_NOTICE, "Got %s signal", "HUP");
 	sighup = true;
 }
 
-static RETSIGTYPE sigint_handler(int a)
-{
-	logger(LOG_NOTICE, _("Got %s signal"), "INT");
+static RETSIGTYPE sigint_handler(int a) {
+	logger(LOG_NOTICE, "Got %s signal", "INT");
 
 	if(saved_debug_level != -1) {
-		logger(LOG_NOTICE, _("Reverting to old debug level (%d)"),
+		logger(LOG_NOTICE, "Reverting to old debug level (%d)",
 			saved_debug_level);
 		debug_level = saved_debug_level;
 		saved_debug_level = -1;
 	} else {
 		logger(LOG_NOTICE,
-			_("Temporarily setting debug level to 5.  Kill me with SIGINT again to go back to level %d."),
+			"Temporarily setting debug level to 5.  Kill me with SIGINT again to go back to level %d.",
 			debug_level);
 		saved_debug_level = debug_level;
 		debug_level = 5;
 	}
 }
 
-static RETSIGTYPE sigalrm_handler(int a)
-{
-	logger(LOG_NOTICE, _("Got %s signal"), "ALRM");
+static RETSIGTYPE sigalrm_handler(int a) {
+	logger(LOG_NOTICE, "Got %s signal", "ALRM");
 	sigalrm = true;
 }
 
-static RETSIGTYPE sigusr1_handler(int a)
-{
+static RETSIGTYPE sigusr1_handler(int a) {
 	dump_connections();
 }
 
-static RETSIGTYPE sigusr2_handler(int a)
-{
+static RETSIGTYPE sigusr2_handler(int a) {
 	dump_device_stats();
 	dump_nodes();
 	dump_edges();
 	dump_subnets();
 }
 
-static RETSIGTYPE sigwinch_handler(int a)
-{
+static RETSIGTYPE sigwinch_handler(int a) {
 	do_purge = true;
 }
 
-static RETSIGTYPE unexpected_signal_handler(int a)
-{
-	logger(LOG_WARNING, _("Got unexpected signal %d (%s)"), a, strsignal(a));
-	cp_trace();
+static RETSIGTYPE unexpected_signal_handler(int a) {
+	logger(LOG_WARNING, "Got unexpected signal %d (%s)", a, strsignal(a));
 }
 
-static RETSIGTYPE ignore_signal_handler(int a)
-{
-	ifdebug(SCARY_THINGS) logger(LOG_DEBUG, _("Ignored signal %d (%s)"), a, strsignal(a));
+static RETSIGTYPE ignore_signal_handler(int a) {
+	ifdebug(SCARY_THINGS) logger(LOG_DEBUG, "Ignored signal %d (%s)", a, strsignal(a));
 }
 
 static struct {
@@ -574,8 +548,7 @@ static struct {
 };
 #endif
 
-void setup_signals(void)
-{
+void setup_signals(void) {
 #ifndef HAVE_MINGW
 	int i;
 	struct sigaction act;
@@ -587,7 +560,7 @@ void setup_signals(void)
 
 	/* Set a default signal handler for every signal, errors will be
 	   ignored. */
-	for(i = 0; i < NSIG; i++) {
+	for(i = 1; i < NSIG; i++) {
 		if(!do_detach)
 			act.sa_handler = SIG_DFL;
 		else
@@ -604,7 +577,7 @@ void setup_signals(void)
 	for(i = 0; sighandlers[i].signal; i++) {
 		act.sa_handler = sighandlers[i].handler;
 		if(sigaction(sighandlers[i].signal, &act, NULL) < 0)
-			fprintf(stderr, _("Installing signal handler for signal %d (%s) failed: %s\n"),
+			fprintf(stderr, "Installing signal handler for signal %d (%s) failed: %s\n",
 					sighandlers[i].signal, strsignal(sighandlers[i].signal),
 					strerror(errno));
 	}
