@@ -1,7 +1,8 @@
 /*
     protocol_edge.c -- handle the meta-protocol, edges
     Copyright (C) 1999-2005 Ivo Timmermans,
-                  2000-2006 Guus Sliepen <guus@tinc-vpn.org>
+                  2000-2009 Guus Sliepen <guus@tinc-vpn.org>
+                  2009      Michael Tokarev <mjt@corpit.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,11 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    $Id: protocol_edge.c 1452 2006-04-26 13:52:58Z guus $
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "system.h"
@@ -36,16 +35,13 @@
 #include "utils.h"
 #include "xalloc.h"
 
-bool send_add_edge(connection_t *c, const edge_t *e)
-{
+bool send_add_edge(connection_t *c, const edge_t *e) {
 	bool x;
 	char *address, *port;
 
-	cp();
-
 	sockaddr2str(&e->address, &address, &port);
 
-	x = send_request(c, "%d %lx %s %s %s %s %lx %d", ADD_EDGE, random(),
+	x = send_request(c, "%d %x %s %s %s %s %x %d", ADD_EDGE, rand(),
 					 e->from->name, e->to->name, address, port,
 					 e->options, e->weight);
 	free(address);
@@ -54,8 +50,7 @@ bool send_add_edge(connection_t *c, const edge_t *e)
 	return x;
 }
 
-bool add_edge_h(connection_t *c)
-{
+bool add_edge_h(connection_t *c) {
 	edge_t *e;
 	node_t *from, *to;
 	char from_name[MAX_STRING_SIZE];
@@ -63,29 +58,21 @@ bool add_edge_h(connection_t *c)
 	char to_address[MAX_STRING_SIZE];
 	char to_port[MAX_STRING_SIZE];
 	sockaddr_t address;
-	long int options;
+	uint32_t options;
 	int weight;
 
-	cp();
-
-	if(sscanf(c->buffer, "%*d %*x "MAX_STRING" "MAX_STRING" "MAX_STRING" "MAX_STRING" %lx %d",
+	if(sscanf(c->buffer, "%*d %*x "MAX_STRING" "MAX_STRING" "MAX_STRING" "MAX_STRING" %x %d",
 			  from_name, to_name, to_address, to_port, &options, &weight) != 6) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s)"), "ADD_EDGE", c->name,
+		logger(LOG_ERR, "Got bad %s from %s (%s)", "ADD_EDGE", c->name,
 			   c->hostname);
 		return false;
 	}
 
 	/* Check if names are valid */
 
-	if(!check_id(from_name)) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s): %s"), "ADD_EDGE", c->name,
-			   c->hostname, _("invalid name"));
-		return false;
-	}
-
-	if(!check_id(to_name)) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s): %s"), "ADD_EDGE", c->name,
-			   c->hostname, _("invalid name"));
+	if(!check_id(from_name) || !check_id(to_name)) {
+		logger(LOG_ERR, "Got bad %s from %s (%s): %s", "ADD_EDGE", c->name,
+			   c->hostname, "invalid name");
 		return false;
 	}
 
@@ -95,6 +82,17 @@ bool add_edge_h(connection_t *c)
 	/* Lookup nodes */
 
 	from = lookup_node(from_name);
+	to = lookup_node(to_name);
+
+	if(tunnelserver &&
+	   from != myself && from != c->node &&
+	   to != myself && to != c->node) {
+		/* ignore indirect edge registrations for tunnelserver */
+		ifdebug(PROTOCOL) logger(LOG_WARNING,
+		   "Ignoring indirect %s from %s (%s)",
+		   "ADD_EDGE", c->name, c->hostname);
+		return true;
+	}
 
 	if(!from) {
 		from = new_node();
@@ -102,16 +100,12 @@ bool add_edge_h(connection_t *c)
 		node_add(from);
 	}
 
-	to = lookup_node(to_name);
-
 	if(!to) {
 		to = new_node();
 		to->name = xstrdup(to_name);
 		node_add(to);
 	}
 
-	if(tunnelserver && from != myself && from != c->node && to != myself && to != c->node)
-		return false;
 
 	/* Convert addresses */
 
@@ -124,12 +118,12 @@ bool add_edge_h(connection_t *c)
 	if(e) {
 		if(e->weight != weight || e->options != options || sockaddrcmp(&e->address, &address)) {
 			if(from == myself) {
-				ifdebug(PROTOCOL) logger(LOG_WARNING, _("Got %s from %s (%s) for ourself which does not match existing entry"),
+				ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) for ourself which does not match existing entry",
 						   "ADD_EDGE", c->name, c->hostname);
 				send_add_edge(c, e);
 				return true;
 			} else {
-				ifdebug(PROTOCOL) logger(LOG_WARNING, _("Got %s from %s (%s) which does not match existing entry"),
+				ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) which does not match existing entry",
 						   "ADD_EDGE", c->name, c->hostname);
 				edge_del(e);
 				graph();
@@ -137,7 +131,7 @@ bool add_edge_h(connection_t *c)
 		} else
 			return true;
 	} else if(from == myself) {
-		ifdebug(PROTOCOL) logger(LOG_WARNING, _("Got %s from %s (%s) for ourself which does not exist"),
+		ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) for ourself which does not exist",
 				   "ADD_EDGE", c->name, c->hostname);
 		e = new_edge();
 		e->from = from;
@@ -167,40 +161,28 @@ bool add_edge_h(connection_t *c)
 	return true;
 }
 
-bool send_del_edge(connection_t *c, const edge_t *e)
-{
-	cp();
-
-	return send_request(c, "%d %lx %s %s", DEL_EDGE, random(),
+bool send_del_edge(connection_t *c, const edge_t *e) {
+	return send_request(c, "%d %x %s %s", DEL_EDGE, rand(),
 						e->from->name, e->to->name);
 }
 
-bool del_edge_h(connection_t *c)
-{
+bool del_edge_h(connection_t *c) {
 	edge_t *e;
 	char from_name[MAX_STRING_SIZE];
 	char to_name[MAX_STRING_SIZE];
 	node_t *from, *to;
 
-	cp();
-
 	if(sscanf(c->buffer, "%*d %*x "MAX_STRING" "MAX_STRING, from_name, to_name) != 2) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s)"), "DEL_EDGE", c->name,
+		logger(LOG_ERR, "Got bad %s from %s (%s)", "DEL_EDGE", c->name,
 			   c->hostname);
 		return false;
 	}
 
 	/* Check if names are valid */
 
-	if(!check_id(from_name)) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s): %s"), "DEL_EDGE", c->name,
-			   c->hostname, _("invalid name"));
-		return false;
-	}
-
-	if(!check_id(to_name)) {
-		logger(LOG_ERR, _("Got bad %s from %s (%s): %s"), "DEL_EDGE", c->name,
-			   c->hostname, _("invalid name"));
+	if(!check_id(from_name) || !check_id(to_name)) {
+		logger(LOG_ERR, "Got bad %s from %s (%s): %s", "DEL_EDGE", c->name,
+			   c->hostname, "invalid name");
 		return false;
 	}
 
@@ -210,36 +192,42 @@ bool del_edge_h(connection_t *c)
 	/* Lookup nodes */
 
 	from = lookup_node(from_name);
-
-	if(!from) {
-		ifdebug(PROTOCOL) logger(LOG_ERR, _("Got %s from %s (%s) which does not appear in the edge tree"),
-				   "DEL_EDGE", c->name, c->hostname);
-		return true;
-	}
-
 	to = lookup_node(to_name);
 
-	if(!to) {
-		ifdebug(PROTOCOL) logger(LOG_ERR, _("Got %s from %s (%s) which does not appear in the edge tree"),
+	if(tunnelserver &&
+	   from != myself && from != c->node &&
+	   to != myself && to != c->node) {
+		/* ignore indirect edge registrations for tunnelserver */
+		ifdebug(PROTOCOL) logger(LOG_WARNING,
+		   "Ignoring indirect %s from %s (%s)",
+		   "DEL_EDGE", c->name, c->hostname);
+		return true;
+	}
+
+	if(!from) {
+		ifdebug(PROTOCOL) logger(LOG_ERR, "Got %s from %s (%s) which does not appear in the edge tree",
 				   "DEL_EDGE", c->name, c->hostname);
 		return true;
 	}
 
-	if(tunnelserver && from != myself && from != c->node && to != myself && to != c->node)
-		return false;
+	if(!to) {
+		ifdebug(PROTOCOL) logger(LOG_ERR, "Got %s from %s (%s) which does not appear in the edge tree",
+				   "DEL_EDGE", c->name, c->hostname);
+		return true;
+	}
 
 	/* Check if edge exists */
 
 	e = lookup_edge(from, to);
 
 	if(!e) {
-		ifdebug(PROTOCOL) logger(LOG_WARNING, _("Got %s from %s (%s) which does not appear in the edge tree"),
+		ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) which does not appear in the edge tree",
 				   "DEL_EDGE", c->name, c->hostname);
 		return true;
 	}
 
 	if(e->from == myself) {
-		ifdebug(PROTOCOL) logger(LOG_WARNING, _("Got %s from %s (%s) for ourself"),
+		ifdebug(PROTOCOL) logger(LOG_WARNING, "Got %s from %s (%s) for ourself",
 				   "DEL_EDGE", c->name, c->hostname);
 		send_add_edge(c, e);	/* Send back a correction */
 		return true;
