@@ -1,7 +1,7 @@
 /*
     device.c -- raw socket
     Copyright (C) 2002-2005 Ivo Timmermans,
-                  2002-2006 Guus Sliepen <guus@tinc-vpn.org>
+                  2002-2009 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,11 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    $Id: device.c 1452 2006-04-26 13:52:58Z guus $
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "system.h"
@@ -29,34 +27,31 @@
 #include "logger.h"
 #include "utils.h"
 #include "route.h"
+#include "xalloc.h"
 
 int device_fd = -1;
-char *device;
-char *iface;
-char ifrname[IFNAMSIZ];
-char *device_info;
+char *device = NULL;
+char *iface = NULL;
+static char ifrname[IFNAMSIZ];
+static char *device_info;
 
 static int device_total_in = 0;
 static int device_total_out = 0;
 
-bool setup_device(void)
-{
+bool setup_device(void) {
 	struct ifreq ifr;
 	struct sockaddr_ll sa;
 
-	cp();
-
-	if(!get_config_string
-		  (lookup_config(config_tree, "Interface"), &iface))
-		iface = "eth0";
+	if(!get_config_string(lookup_config(config_tree, "Interface"), &iface))
+		iface = xstrdup("eth0");
 
 	if(!get_config_string(lookup_config(config_tree, "Device"), &device))
-		device = iface;
+		device = xstrdup(iface);
 
-	device_info = _("raw socket");
+	device_info = "raw socket";
 
 	if((device_fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0) {
-		logger(LOG_ERR, _("Could not open %s: %s"), device_info,
+		logger(LOG_ERR, "Could not open %s: %s", device_info,
 			   strerror(errno));
 		return false;
 	}
@@ -65,7 +60,7 @@ bool setup_device(void)
 	strncpy(ifr.ifr_ifrn.ifrn_name, iface, IFNAMSIZ);
 	if(ioctl(device_fd, SIOCGIFINDEX, &ifr)) {
 		close(device_fd);
-		logger(LOG_ERR, _("Can't find interface %s: %s"), iface,
+		logger(LOG_ERR, "Can't find interface %s: %s", iface,
 			   strerror(errno));
 		return false;
 	}
@@ -76,30 +71,27 @@ bool setup_device(void)
 	sa.sll_ifindex = ifr.ifr_ifindex;
 
 	if(bind(device_fd, (struct sockaddr *) &sa, (socklen_t) sizeof(sa))) {
-		logger(LOG_ERR, _("Could not bind %s to %s: %s"), device, iface, strerror(errno));
+		logger(LOG_ERR, "Could not bind %s to %s: %s", device, iface, strerror(errno));
 		return false;
 	}
 
-	logger(LOG_INFO, _("%s is a %s"), device, device_info);
+	logger(LOG_INFO, "%s is a %s", device, device_info);
 
 	return true;
 }
 
-void close_device(void)
-{
-	cp();
-
+void close_device(void) {
 	close(device_fd);
+
+	free(device);
+	free(iface);
 }
 
-bool read_packet(vpn_packet_t *packet)
-{
+bool read_packet(vpn_packet_t *packet) {
 	int lenin;
 
-	cp();
-
 	if((lenin = read(device_fd, packet->data, MTU)) <= 0) {
-		logger(LOG_ERR, _("Error while reading from %s %s: %s"), device_info,
+		logger(LOG_ERR, "Error while reading from %s %s: %s", device_info,
 			   device, strerror(errno));
 		return false;
 	}
@@ -108,21 +100,18 @@ bool read_packet(vpn_packet_t *packet)
 
 	device_total_in += packet->len;
 
-	ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Read packet of %d bytes from %s"), packet->len,
+	ifdebug(TRAFFIC) logger(LOG_DEBUG, "Read packet of %d bytes from %s", packet->len,
 			   device_info);
 
 	return true;
 }
 
-bool write_packet(vpn_packet_t *packet)
-{
-	cp();
-
-	ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Writing packet of %d bytes to %s"),
+bool write_packet(vpn_packet_t *packet) {
+	ifdebug(TRAFFIC) logger(LOG_DEBUG, "Writing packet of %d bytes to %s",
 			   packet->len, device_info);
 
 	if(write(device_fd, packet->data, packet->len) < 0) {
-		logger(LOG_ERR, _("Can't write to %s %s: %s"), device_info, device,
+		logger(LOG_ERR, "Can't write to %s %s: %s", device_info, device,
 			   strerror(errno));
 		return false;
 	}
@@ -132,11 +121,8 @@ bool write_packet(vpn_packet_t *packet)
 	return true;
 }
 
-void dump_device_stats(void)
-{
-	cp();
-
-	logger(LOG_DEBUG, _("Statistics for %s %s:"), device_info, device);
-	logger(LOG_DEBUG, _(" total bytes in:  %10d"), device_total_in);
-	logger(LOG_DEBUG, _(" total bytes out: %10d"), device_total_out);
+void dump_device_stats(void) {
+	logger(LOG_DEBUG, "Statistics for %s %s:", device_info, device);
+	logger(LOG_DEBUG, " total bytes in:  %10d", device_total_in);
+	logger(LOG_DEBUG, " total bytes out: %10d", device_total_out);
 }
