@@ -1,7 +1,7 @@
 /*
     device.c -- Interaction with Solaris tun device
     Copyright (C) 2001-2005 Ivo Timmermans,
-                  2001-2006 Guus Sliepen <guus@tinc-vpn.org>
+                  2001-2009 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,11 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    $Id: device.c 1452 2006-04-26 13:52:58Z guus $
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 
@@ -31,30 +29,28 @@
 #include "logger.h"
 #include "net.h"
 #include "utils.h"
+#include "xalloc.h"
 
 #define DEFAULT_DEVICE "/dev/tun"
 
 int device_fd = -1;
 char *device = NULL;
 char *iface = NULL;
-char *device_info = NULL;
+static char *device_info = NULL;
 
 static int device_total_in = 0;
 static int device_total_out = 0;
 
-bool setup_device(void)
-{
+bool setup_device(void) {
 	int ip_fd = -1, if_fd = -1;
 	int ppa;
 	char *ptr;
 
-	cp();
-
 	if(!get_config_string(lookup_config(config_tree, "Device"), &device))
-		device = DEFAULT_DEVICE;
+		device = xstrdup(DEFAULT_DEVICE);
 
 	if((device_fd = open(device, O_RDWR | O_NONBLOCK)) < 0) {
-		logger(LOG_ERR, _("Could not open %s: %s"), device, strerror(errno));
+		logger(LOG_ERR, "Could not open %s: %s", device, strerror(errno));
 		return false;
 	}
 
@@ -66,63 +62,60 @@ bool setup_device(void)
 	ppa = atoi(ptr);
 
 	if((ip_fd = open("/dev/ip", O_RDWR, 0)) < 0) {
-		logger(LOG_ERR, _("Could not open /dev/ip: %s"), strerror(errno));
+		logger(LOG_ERR, "Could not open /dev/ip: %s", strerror(errno));
 		return false;
 	}
 
 	/* Assign a new PPA and get its unit number. */
 	if((ppa = ioctl(device_fd, TUNNEWPPA, ppa)) < 0) {
-		logger(LOG_ERR, _("Can't assign new interface: %s"), strerror(errno));
+		logger(LOG_ERR, "Can't assign new interface: %s", strerror(errno));
 		return false;
 	}
 
 	if((if_fd = open(device, O_RDWR, 0)) < 0) {
-		logger(LOG_ERR, _("Could not open %s twice: %s"), device,
+		logger(LOG_ERR, "Could not open %s twice: %s", device,
 			   strerror(errno));
 		return false;
 	}
 
 	if(ioctl(if_fd, I_PUSH, "ip") < 0) {
-		logger(LOG_ERR, _("Can't push IP module: %s"), strerror(errno));
+		logger(LOG_ERR, "Can't push IP module: %s", strerror(errno));
 		return false;
 	}
 
 	/* Assign ppa according to the unit number returned by tun device */
 	if(ioctl(if_fd, IF_UNITSEL, (char *) &ppa) < 0) {
-		logger(LOG_ERR, _("Can't set PPA %d: %s"), ppa, strerror(errno));
+		logger(LOG_ERR, "Can't set PPA %d: %s", ppa, strerror(errno));
 		return false;
 	}
 
 	if(ioctl(ip_fd, I_LINK, if_fd) < 0) {
-		logger(LOG_ERR, _("Can't link TUN device to IP: %s"), strerror(errno));
+		logger(LOG_ERR, "Can't link TUN device to IP: %s", strerror(errno));
 		return false;
 	}
 
 	if(!get_config_string(lookup_config(config_tree, "Interface"), &iface))
-		asprintf(&iface, "tun%d", ppa);
+		xasprintf(&iface, "tun%d", ppa);
 
-	device_info = _("Solaris tun device");
+	device_info = "Solaris tun device";
 
-	logger(LOG_INFO, _("%s is a %s"), device, device_info);
+	logger(LOG_INFO, "%s is a %s", device, device_info);
 
 	return true;
 }
 
-void close_device(void)
-{
-	cp();
-
+void close_device(void) {
 	close(device_fd);
+
+	free(device);
+	free(iface);
 }
 
-bool read_packet(vpn_packet_t *packet)
-{
+bool read_packet(vpn_packet_t *packet) {
 	int lenin;
 
-	cp();
-
 	if((lenin = read(device_fd, packet->data + 14, MTU - 14)) <= 0) {
-		logger(LOG_ERR, _("Error while reading from %s %s: %s"), device_info,
+		logger(LOG_ERR, "Error while reading from %s %s: %s", device_info,
 			   device, strerror(errno));
 		return false;
 	}
@@ -138,7 +131,7 @@ bool read_packet(vpn_packet_t *packet)
 			break;
 		default:
 			ifdebug(TRAFFIC) logger(LOG_ERR,
-					   _ ("Unknown IP version %d while reading packet from %s %s"),
+					   "Unknown IP version %d while reading packet from %s %s",
 					   packet->data[14] >> 4, device_info, device);
 			return false;
 	}
@@ -147,21 +140,18 @@ bool read_packet(vpn_packet_t *packet)
 
 	device_total_in += packet->len;
 
-	ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Read packet of %d bytes from %s"), packet->len,
+	ifdebug(TRAFFIC) logger(LOG_DEBUG, "Read packet of %d bytes from %s", packet->len,
 			   device_info);
 
 	return true;
 }
 
-bool write_packet(vpn_packet_t *packet)
-{
-	cp();
-
-	ifdebug(TRAFFIC) logger(LOG_DEBUG, _("Writing packet of %d bytes to %s"),
+bool write_packet(vpn_packet_t *packet) {
+	ifdebug(TRAFFIC) logger(LOG_DEBUG, "Writing packet of %d bytes to %s",
 			   packet->len, device_info);
 
 	if(write(device_fd, packet->data + 14, packet->len - 14) < 0) {
-		logger(LOG_ERR, _("Can't write to %s %s: %s"), device_info,
+		logger(LOG_ERR, "Can't write to %s %s: %s", device_info,
 			   device, strerror(errno));
 		return false;
 	}
@@ -171,11 +161,8 @@ bool write_packet(vpn_packet_t *packet)
 	return true;
 }
 
-void dump_device_stats(void)
-{
-	cp();
-
-	logger(LOG_DEBUG, _("Statistics for %s %s:"), device_info, device);
-	logger(LOG_DEBUG, _(" total bytes in:  %10d"), device_total_in);
-	logger(LOG_DEBUG, _(" total bytes out: %10d"), device_total_out);
+void dump_device_stats(void) {
+	logger(LOG_DEBUG, "Statistics for %s %s:", device_info, device);
+	logger(LOG_DEBUG, " total bytes in:  %10d", device_total_in);
+	logger(LOG_DEBUG, " total bytes out: %10d", device_total_out);
 }
